#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

Config - Configuration control 

=head1 Description

This class is used for configuration control for WDB.
When creating a new Config object it will automatically detect OS system, and
find the configuration data.


=head1 Public Methods
=cut#--------------------------------------------------------------------------

package WDB::Config;

use English;                    	# Use long "english" variable names 

$DEFAULT_CONFIG_PATH_UNIX    = '/etc/bass/config';
$DEFAULT_CONFIG_PATH_WINDOWS = 'c:/BassWebEDI/config';

my(%CONFIG);
$CONFIG_PATH = '';

#$debug = $main::debug;

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 GuessOS()

Guesses the operating system type. 
This is a class method and should be called like : 

    $os = WDB::Config::GuessOS();

Returns 'Windows' or 'Unix'.

=cut
#------------------------------------------------------------------------------
sub GuessOS
{
    my($os);

    #-----------------------------------
    # Guess OS
    # TODO: Will this always work ?
    #-----------------------------------
    if ( $ENV{OS} =~ m/Windows/ ) {
	$os = 'Windows';
    } else {
	$os = 'Unix';
    }

    return $os;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 new( name )

New creates a new config object.

=cut
#------------------------------------------------------------------------------
sub new 
{
    my($type) = shift;
    my($name) = @_;

    #-----------------------------------
    # Chached config ?
    #-----------------------------------
    return $CONFIG{$name} if ( $CONFIG{$name} ) ;

    #-----------------------------------
    # No. Make new config object.
    #-----------------------------------
    my($self) = {};
    bless ($self, $type);

    $self->{OS} = GuessOS();
    $self->{name} = $name;

    #-----------------------------------
    # Get base path for config files
    #-----------------------------------
    if ( $self->{OS} eq 'Windows' ) {
	$CONFIG_PATH = $DEFAULT_CONFIG_PATH_WINDOWS;
    } else {
	$CONFIG_PATH = $DEFAULT_CONFIG_PATH_UNIX;
    }

    #-----------------------------------
    # Load config.
    #-----------------------------------

    my($filename) = $CONFIG_PATH . '/default_'.$name.'.pm';

    open(DEFAULT, $filename) || die "Error reading $filename";
    my($default) = eval ( join('',<DEFAULT>) );
    if ( $@ ) {
	die( "Error evaluating $filename \n $@\n" );
    }
    close(DEFAULT);

    my($param);
    foreach $param ( @$default ) {
	#printf "Default: %10s = %s\n", $param->{name}, $param->{value};
	$self->{param}{$param->{name}} = $param->{value};
    }

    $filename = $CONFIG_PATH . '/config_'.$name.'.bwe';
    if ( open(FILE, $filename) ) {
	my($key,$val);
	while (<FILE>) {
	    next if /^#.*$/;
	    #( $key, $val ) = split('=', $_, 2);
	    #chomp($val);
	    ($key, $val) = /^(\w+)\s*=\s*(.*)\s*$/;
	    $val =~ s/\s*$//;
	    $self->{param}{$key} = $val;
	}
	close(FILE);
    }

    #-----------------------------------
    # Cache config and return.
    #-----------------------------------
    $CONFIG{$name} = $self;
    return $self;
}

sub getVal
{
    my($self) = shift;
    my($param) = @_;
    return $self->{$param};
}

sub setVal
{
    my($self) = shift;
    my($param,$val) = @_;
    $self->{$param} = $val;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 TEST()

TEST of the WDB::Config module.

Run from commandline: 

    perl -w -e "require (WDB::Config); WDB::Config::TEST()"

=cut
#------------------------------------------------------------------------------
sub TEST 
{
    printf "===============================================\n";
    printf "Testing WDB::Config\n";
    printf "===============================================\n";
    printf "OS : %s\n", GuessOS();

    #$config = new WDB::Config('test');
    $config = new WDB::Config('kp');
    printf $config->dumpParams();
}

sub dumpParams
{
    my($self) = shift;
    my ($string) = "\nConfiguration parameters for: $self->{name}\n\n";

    foreach $key ( sort keys %{$self->{param}} ) {
	$string .= sprintf "%-16s = '%s'\n", $key, $self->{param}{$key};
    }

    return $string;
}
1;
