#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

DBI_Oracle -- Database Interface for Oracle.

=head1 Description

This class defines the database interface that WDB uses to access 
Oracle databases.

=head1 Public Methods
=cut#--------------------------------------------------------------------------

use WDB::DBI;

package WDB::DBI_Oracle;@ISA = qw( WDB::DBI );
@WDB::DBI_Oracle::Inherit::ISA = @ISA;	# Access to overridden methods.
use English;                    	# Use long "english" variable names 
use Oraperl;			# Get the Oraperl interface to DBD::Oracle

#$debug = $main::debug;
#$ora_cache = 500;
#$ora_trunc = 4000;		# Make sure we don't truncate those long fields.
#$ora_long  = 4000;		#
#$ora_debug = 0;			#


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 new( user, pswd, server, database )

New logs in to a database and creates a new database connection object.

=cut
#------------------------------------------------------------------------------
sub new 
{
    my($type) = shift;
    my($self) = new WDB::DBI(@_); # First run the DBI constructor...
    bless $self, $type;		  # Then re-bless so we are a DBI_Oracle object.


    $self->{dbproc} = &ora_login($self->{database}, $self->{user}, $self->{pswd}) 
        || WDB::Error::Fatal($ora_errstr);

    $self->{dbproc} != -1 || WDB::Error::Fatal("Can't connect to server $self->{server} ..\n");

    &ora_autocommit( $self->{dbproc}, 1 );

    return $self;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 do( sql_statement )

Executes the given SQL statement. This is for I<one-shot> SQL statements.
For selects use the open/fetch/close methods.

Dies on error !

=cut
#------------------------------------------------------------------------------
sub do
{
    my($self) = shift;
    my($sql)  = @_;

    print main::DEBUG "DBI_Oracle::do( $sql ) \n" if $main::debug;
    $self->{csr} = &ora_do($self->{dbproc}, $sql) || WDB::Error::Fatal("$ora_errstr : $sql");
}
 

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 try( sql_statement )

Same as do, but doesn't die on error. Instead true or false is returned.

=cut
#------------------------------------------------------------------------------
sub try
{
    my($self) = shift;
    my($sql)  = @_;

    print main::DEBUG "DBI_Oracle::try( $sql ) \n" if $main::debug;
    #if ( ($self->{csr} = &ora_do($self->{dbproc}, $sql)) ) {
    #	return 1;
    #}
    #return 0;
    #return 1;

    return &ora_do($self->{dbproc}, $sql);
}
 

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 open( sql_statement )

Prepares and executes the given SQL select statement.
The returned rows can be retrieved via the fetch method.

Dies on error !

=cut
#------------------------------------------------------------------------------
sub open
{
    my($self) = shift;
    my($sql)  = @_;

    print main::DEBUG "DBI_Oracle::open( $sql ) \n" if $main::debug;
    $self->{csr} = &ora_open($self->{dbproc}, $sql) 
       || WDB::Error::Fatal("$ora_errstr : $sql") ;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 prepare( sql_statement )

Prepares and the given SQL select statement.
The returned rows can be retrieved via the fetch method.

Dies on error !

=cut
#------------------------------------------------------------------------------
sub prepare
{
    my($self) = shift;
    my($sql)  = @_;

    print main::DEBUG "DBI_Oracle::prepare( $sql ) \n" if $main::debug;
    $self->{csr} = &ora_open($self->{dbproc}, $sql) 
       || WDB::Error::Fatal("$ora_errstr : $sql") ;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 bind( params, ... )

    Binds the given parameters to the previously prepared SQL statement.

Dies on error !

=cut
#------------------------------------------------------------------------------
sub bind
{
    my($self) = shift;
    #my($sql)  = @_;

    if ($main::debug) {
    	print main::DEBUG "DBI_Oracle::bind( $sql ) \n" ;
        my($i,$val);
        foreach $val (@_) {
	    $i++;
    	    print main::DEBUG "- bind :".$i." = ".$val." \n" ;
	}
    }
    &ora_bind($self->{csr}, @_) 
       || WDB::Error::Fatal("$ora_errstr : bind error?") ;
    #   || return 0;
    return 1;
}
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 bind_param( param_no, param_val )

Binds the given parameter to the previously prepared SQL statement.

Dies on error !

=cut
#------------------------------------------------------------------------------
sub bind_param
{
    my($self) = shift;
    #my($no,$val)  = @_;

    print main::DEBUG "DBI_Oracle::bind_param(".join(',',@_).") \n" if $main::debug;

    $self->{csr}->bind_param(@_) 
       || WDB::Error::Fatal("$ora_errstr : bind error?") ;
    return 1;
}



#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 getLastError()

Returns last error. 

Usage:
    ($err_code,$native_err_num, $native_err_msg) = $dbi->getLastError();

    NB! In the Oracle version error codes are global! - So the error could
    from another database connection.

=cut
#------------------------------------------------------------------------------
sub getLastError
{
    my($self) = shift;
    my($ErrNum, $ErrText, $ErrConn) = ($ora_errno, $ora_errstr );
    my($ErrCode) = $WDB::DBI::ERR_UNKNOWN;

    $ErrCode = $WDB::DBI::ERR_UNIQ if ( $ErrNum == 1 ) ;

    return($ErrCode, $ErrNum, $ErrText); 
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 close()

Closes a previously open'ed statement.

=cut
#------------------------------------------------------------------------------
sub close
{
    my($self) = shift;

    &ora_close($self->{csr}) || WDB::Error::Fatal($ora_errstr);
}
 

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 nextrow()

Gets the next row from a previous select (dbi_dosql).

After a open("select ... ") call, this function can be 
called repeatetly to retrieve all rows of the query.

Example :
	$dbi->open("select * from mytab");
	while( %row = $dbi->nextrow ) {
	    print $row{'columnname'};
	    ...
	}

Returns	an associative array (keyed on the column name) of formatted 
data, based on the datatype of the corresponding columns.

=cut
#------------------------------------------------------------------------------
sub nextrow
{
    my($self) = shift;
    my(%row);

    my(@titles, @trow, $i) ;

    @trow = &ora_fetch($self->{csr});
    if (@trow) {
	@titles = &ora_titles($self->{csr}) ;
	undef %row;
	for ($i = 0; $i < @titles; ++$i) {
            $row{$titles[$i]} = $trow[$i] ;
	}
    } else {
        #print "DBI_Oracle::nextrow: *** No rows ***\n";
	#%row = undef ;
	undef %row;
	#return undef;
    }

    print main::DEBUG "DBI_Oracle::nextrow=".join(',',%row)."\n" if $main::debug;

    print "($ora_errno) $ora_errstr" if $ora_errno;
    return %row ;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 disconnect()

Disconnects the current database connection

After this function is called there are no current database 
connection => no other function from this package can be 
called. 

This method is automatically called when the current object is
destroyed.

=cut
#------------------------------------------------------------------------------
sub disconnect
{
    my($self) = shift;

    &ora_logoff($self->{dbproc}) || WDB::Error::Fatal($ora_errstr);
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 commit

Commits SQL statements executed since last commit

=cut
#------------------------------------------------------------------------------
sub commit
{
    my($self) = shift;

    &ora_commit( $self->{dbproc} );
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 rollback

Rollsback SQL statements executed since last commit

=cut
#------------------------------------------------------------------------------
sub rollback
{
    my($self) = shift;

    &ora_rollback( $self->{dbproc} );
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 autocommit( on_or_off )

Enables automatic commit after each executed SQL statement.

If I<on_or_off> is true (non-zero) autocommit is enabled. If it is
false (zero) is is disabled.

=cut
#------------------------------------------------------------------------------
sub autocommit
{
    my($self)  = shift;
    my($onoff) = shift;

    &ora_autocommit( $self->{dbproc}, $onoff );
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 dateformat( format )

Sets the default output format for dates.

Sets the order of the date parts month/day/year for entering
datetime data. Valid arguments are mdy, dmy, ymd, ydm, myd, dym.

=cut
#------------------------------------------------------------------------------
sub dateformat
{
    my($self)   = shift;
    my($format) = shift;
    
    $self->do("ALTER SESSION SET NLS_DATE_FORMAT='$format'");
}


1;#			    ----- EOF -----
