#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

Error - Collects and hanldes error messages.

=head1 Description

This package can collect error messages during the run-time of the program, 
and output them upon program termination in a number of different ways. 
To an email address, to a file, or to stdout/stderr.

	#---------------------------------------------
	require 'Error';

	WDB::Error::Logfile( 'filename' );
	WDB::Error::MailAddr( 'email@host.dk' );
	WDB::Error::NoStdout();	    # Disables terminal output

	WDB::Error::Error( $msg );
	WDB::Error::Fatal( $msg ); # Dies...
	.....
	#---------------------------------------------

=head1 Public Methods
=cut
#------------------------------------------------------------------------------
use strict ;
no strict "vars";

package WDB::Error;
use English;                    # Use long "english" variable names 

$MAILPROG = "/usr/lib/sendmail";

$mailAddr = '';
$logFile  = '';
$noStdout = 0;
$errors   = '';
$syslog   = '';

END { &print_errors };

sub Error
{
    my( $msg ) = @_;

    $errors .= $msg . "\n";
}

sub Fatal
{
    my( $msg ) = @_;

    $errors .= $msg . "\n";
    exit 0;
}

sub Log
{
    my( $msg ) = @_;

    $syslog .= $msg . "\n";
}

sub Logfile
{
    my( $file ) = @_;

    $logFile = $file;
}

sub MailAddr
{
    my( $addr ) = @_;

    $mailAddr = $addr;
}

sub NoStdout
{
    $noStdout = 1;
}

sub print_errors
{
    return unless $errors or $syslog;
    my( $sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst ) = localtime( time() );
    my( $today ) = sprintf "%02d-%02d-%02d %02d:%02d:%02d",$year+1900,$mon+1,$mday,$hour,$min,$sec;

    if ( $errors ) {
	print "Content-type: text/html\n\n$errors\n" unless $noStdout;
    }

    if ( $mailAddr ) {
	open( MAIL, "| $MAILPROG $mailAddr" ); 
	print MAIL "Subject: WDB Error\n";
	print MAIL "An error has occured while running WDB:\n";
	print MAIL "=======================================\n";
	print MAIL $errors;
	print MAIL "---\n";
	print MAIL $syslog;

	print MAIL "\n---------------\nCGI\n---------------\n";
	foreach $key ( keys ( %main::in ) ) {
	    print MAIL "$key=$main::in{$key} \n";
	}

	print MAIL "\n---------------\nEnvironment\n---------------\n";
	foreach $env ( sort keys(%ENV) ) {
	    printf MAIL "$env=$ENV{$env}\n";
	}

	close( MAIL );
    }

    if ( $logFile ) {
	open( FILE, ">> $logFile" ); 
	print FILE "===============================================\n";
	print FILE "$today\n";
	print FILE "-----------------------------------------------\n";
	print FILE $errors;
	print FILE "---\n";
	print FILE $syslog;

	print FILE "\n---------------\nCGI\n---------------\n";
	foreach $key ( keys ( %main::in ) ) {
	    print FILE "$key=$main::in{$key} \n";
	}

	print FILE "\n---------------\nEnvironment\n---------------\n";
	foreach $env ( sort keys(%ENV) ) {
	    print FILE "$env=$ENV{$env}\n";
	}

	close( FILE );
    }
}

1;
