#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

Eval -- Environment for doing eval's in WDB.

=head1 Description

This package provides the environment for doing eval's in WDB.
All attributes etc that are evaluated (like to_db, from_db, etc.),
are run in this package. 

The Eval environment provides the following variables :

%form	: Assoc. array of the FDF form attributes.
%field	: Assoc. array of the FDF field attributes. These are accessed like this

	    $val = %field{FIELD,attribute}

	  Where FIELD is the name of the field (Given by FIELD= in the FDF)
	  and attribute is on of the attributes of the field. There are also
	  a set of special attributes available :

	      value  :	The current value of the field.
	      query  :	The current query value of the field
	      oper   :	The current query operand on the field

=head1 Public Methods
=cut#--------------------------------------------------------------------------

package WDB::FieldHash;

*in= %main::in;

sub TIEHASH 
{
    my($type) = shift;
    my($self) = {};
    my($fdf) = @_;
    bless ($self, $type);

    $self->{fdf} = $fdf;

    return $self;
}

sub FETCH 
{
    my($self) = shift;
    my($key)  = shift;

    my($field,$attr) = split($;, $key);
    $attr = 'value' unless $attr;
    
    #print "Field: FETCH $key\n";
    #print "Field: ..... $field - $attr\n";

    return ( $self->{fdf}->{field}->{$field}->{$attr} );
}

sub STORE 
{
    my($self) = shift;
    my($key)  = shift;
    my($value)= shift;

    #my($field,$attr) = split($;, $key);
    my($field,$attr) = split(',', $key);
    $attr = 'value' unless $attr;

    #print main::DEBUG "Field: STORE $key\n" if $main::debug;
    #print main::DEBUG "Field: ..... $field - $attr = $value\n" if $main::debug;
    
    $self->{fdf}->{field}->{$field}->set_oper('=') if $attr eq 'value';

    return ( $self->{fdf}->{field}->{$field}->{$attr} = $value );
}

sub EXISTS
{
    my($self) = shift;
    my($key)  = shift;
    
    return ( defined $self->{fdf}->{field}->{$key} );
}

###########################
##### NOT IMPLEMENTED #####
###########################
# FIRSTKEY this
# NEXTKEY this, lastkey
# DELETE this, key
# DESTROY this
###########################


package WDB::FormHash;

sub TIEHASH 
{
    my($type) = shift;
    my($self) = {};
    my($fdf) = @_;
    bless ($self, $type);

    $self->{fdf} = $fdf;

    return $self;
}

sub FETCH 
{
    my($self) = shift;
    my($key)  = shift;
    
    return ( $self->{fdf}->{$key} );
}

sub STORE 
{
    my($self) = shift;
    my($key)  = shift;
    my($value)= shift;
    
    return ( $self->{fdf}->{$key} = $value );
}

sub EXISTS
{
    my($self) = shift;
    my($key)  = shift;
    
    return ( defined $self->{fdf}->{$key} );
}

###########################
##### NOT IMPLEMENTED #####
###########################
# FIRSTKEY this
# NEXTKEY this, lastkey
# DELETE this, key
# DESTROY this
###########################


package WDB::Eval;
#require WDB::FormHash;
#require WDB::FieldHash;
use English;                    	# Use long "english" variable names 

# Make some CGI variables available.

$REMOTE_HOST	 = $ENV{REMOTE_HOST};
$REMOTE_USER	 = $ENV{REMOTE_USER};
$SERVER_NAME	 = $ENV{SERVER_NAME};
$SERVER_PORT	 = $ENV{SERVER_PORT};
$HTTP_USER_AGENT = $ENV{HTTP_USER_AGENT};
$HTTP_HOST   	 = ( $ENV{HTTP_HOST} 
		     ? $ENV{HTTP_HOST} 
		     : $ENV{SERVER_NAME} . ( ($ENV{SERVER_PORT}==80)
					     ? ''
					     : ':'.$ENV{SERVER_PORT})
		    );

($BROWSER,$BROWSER_VER) = $HTTP_USER_AGENT =~ /([^\s\/]+)\/?([0-9\.]+)?/;

my($fdf);

sub init
{
    $fdf = shift;

    $cgi=$fdf->{cgi};

    tie( %WDB::Eval::form,  WDB::FormHash,  $fdf );
    tie( %WDB::Eval::field, WDB::FieldHash, $fdf );
}
    
#
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE	Evaluate an attribute using 'eval' - with error handling etc.
#
#.REMARKS	Dies on error.
#
#.RETURNS	The return value of the 'eval'
#------------------------------------------------------------------------------
sub eval
{
    my($msg, $val) = @_;
    my($ret);

    $ret = eval $val;
    if ( $@ ) {
	WDB::Error::Fatal( "Error evaluating $msg :<p>\n $@\n"
			 . "Please report this error.\n"
			 . "Exiting...\n");
    }

    return $ret;
}

#
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE	Evaluate an attribute using 'eval' - with error handling etc.
#
#.REMARKS	Dies on error.
#
#		This is a TEST
#
#.RETURNS	The return value of the 'eval'
#------------------------------------------------------------------------------
sub eval_fdf
{
    my($msg, $val, $fdf) = @_;
    my($ret);

    #print main::DEBUG "*** EVAL_FDF TEST ***\n" if $main::debug;

    tie( %WDB::Eval::form,  WDB::FormHash,  $fdf );
    tie( %WDB::Eval::field, WDB::FieldHash, $fdf );

    #print main::DEBUG "*** FDF NAME    = $form{NAME}\n" if $main::debug;
    #print main::DEBUG "*** Eval string = $val\n" if $main::debug;

    $ret = eval $val;
    if ( $@ ) {
	WDB::Error::Fatal( "Error evaluating $msg :<p>\n $@\n"
			 . "Please report this error.\n"
			 . "Exiting...\n");
    }

    return $ret;
}

sub wdb_action
{
    #$fdf->{ldf}->wdb_ACTION(@_);
    # Temp. hack : Moved wdb_ACTION to Fdf.pm - can now be called without 
    # first initializing the Layout (Ldf)
    $fdf->wdb_ACTION(@_);
}


1;#		--------- The End --------
