#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

Fdf - Form Definition File for WDB

=head1 Description

This class defines the forms used in WDB. A Form is a collection of fields
in the database, plus descriptions etc....

=head1 Private Data

    $self->{'fields'} 		# List of all fields in order.
    $self->{'keys'} 		# List of key fields
    $self->{'field'}{...}	# Fields. (Type= WDB::Field)
    $self->{...}		# Form attributes
    $self->{'maxlabel'}		# Max length of field label strings
    $self->{'ldf'}		# Initialized LDF object (WDB::Ldf)

=head1 Public Methods
=cut#--------------------------------------------------------------------------

use WDB::Ldf;
#use WDB::Field_char;

package  WDB::Fdf;
use English;                    # Use long "english" variable names 

$debug = $main::debug;


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 new( filename )

New creates a new Fdf object 

=cut
#------------------------------------------------------------------------------
sub new 
{
    my($type) = shift;
    my($self) = {};
    my($formdir,$cgi) = @_;
    my($filename);

    bless ($self, $type);

    $self->{'cgi'} = $cgi;

    if ( $cgi->param('wdb_fdf') ) {
	$filename = $cgi->param('wdb_fdf');
	$filename =~ s/:/\//g;
	$filename = $formdir . '/' . $filename . '.fdf';
	$self->parse_file($filename);
    }

    return $self;
}

#
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE	Parse a form definition file ('fdf' file)
#
#.REMARKS
#
#.RETURNS	True, or not at all ....
#------------------------------------------------------------------------------
sub parse_file 
{
    my($self) = shift;
    my($file) = @_;
    my(%field,$old_attr_type,$old_attr_key,$key,$void,$val,$current);

    $self->{'fields'} = [];
    $self->{'keys'} = [];

    open( FDF, $file ) || WDB::Error::Fatal("Can't open $file: $!\n");

    $lineno = 0;
    while ( <FDF> ) {	  
	$lineno++;

	#-----------------------------------------------------------------
	# Parse a line of the form : "key=value". Comments and unnessesary
	# blanks are skipped. (blanks in the middel of values are kept!)
	# The key should start at the 1. character on the line - lines 
	# starting with whitespace are interpreted as continuation lines.
	#-----------------------------------------------------------------
	next if /^#.*$/;
	($key, $void, $val ) = 
		/^(\w+)\s*(=\s*((\s*\".*\"|\s*[^#\s\n]+)+))?\s*(#.*)?$/;

	if ( ! $key ) {   # Continuation line ...
	    ($val) = /^\s*((\s*[^#\s\n]+)+)\s*(#.*)?$/;
	    next unless $val;
	    if ( $old_attr_type eq 'form' ) {
		$self->{$old_attr_key} .= "\n$val";
	    } else {
		$field{$old_attr_key} .= "\n$val";
	    }
	    next;
	}

	#--------------------------------------------------
	# Begining of a new FIELD definition ?
	#--------------------------------------------------
	if ( $key eq "FIELD" ) {
	    if ( $current ) {	# Create new Field object.
		# Default value for column is the field name....
		$field{column} = $field{FIELD} unless $field{column};
		require 'WDB/Field_'.$field{type}.'.pm';
		$self->{field}{$current} = 
			 eval('new WDB::Field_'.$field{type}.'($self,%field)');
		WDB::Error::Fatal("Error creating field of type $field{type} at line $lineno: $@") if $@;
		push( @{$self->{fields}}, $self->{field}{$current} );
	    }
	    $current = $val;
	    undef %field;
	    $field{FIELD} = $val;
	    next;
	}

	#--------------------------
	# Is it a FORM attribute ?
	#--------------------------
	if ( ! $current ) {	  # Defs' before first field are form defs'
	    $self->{$key} = $val;	
	    $old_attr_key = $key;
	    $old_attr_type = 'form';
	    next;
	}

	#--------------------------
	# It's a FIELD attribute.
	#--------------------------

	if ( $key eq "key" ) {
	    push( @{$self->{'keys'}}, $current);
	}
	$val = 1 unless defined $val;
	$field{$key}   = $val;
	$old_attr_key  = $key;
	$old_attr_type = 'field';
    }

    #--------------------------------
    # Now create the last field....
    #--------------------------------
    if ( $current ) {	
	$field{column} = $field{FIELD} unless $field{column};
	require 'WDB/Field_'.$field{type}.'.pm';
	$self->{field}{$current} = 
		 eval('new WDB::Field_'.$field{type}.'($self,%field)');
	WDB::Error::Fatal("Error creating field of type $field{type} at line $lineno : $@") if $@;
	push( @{$self->{fields}}, $self->{field}{$current} );
    }

    #-------------------------------------------------------------------
    # Find the length of the longest labelstring for formating purposes
    #-------------------------------------------------------------------
    $self->{'maxlabel'} = 0;
    foreach $f ( @{$self->{fields}} ) {
	$self->{'maxlabel'} = length($f->{'label'}) 
		if length($f->{'label'}) > $self->{'maxlabel'};
    }

    #--------------------------------------------------------
    # If a 'PERL' attribute exists in the form : Evaluate it
    #--------------------------------------------------------
    if ( $self->{'PERL'} ) {
	WDB::Eval::eval_fdf('PERL',$self->{'PERL'},$self);
    }

    #-------------------------------------------------------------
    # Use default database from wdb.conf if not specified in FDF.
    #-------------------------------------------------------------
    $self->{'DATABASE'} = $WDB::Eval::database unless $self->{'DATABASE'};
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 layout( layout )

Initialize layout.

=cut
#------------------------------------------------------------------------------
sub layout 
{
    my($self) = shift;
    my($layout) = @_;
    my($ldf);

    print main::DEBUG "FDF:Using layout $layout\n" if $main::debug;
    $ldf = new WDB::Ldf($self->{LDF},$self) || WDB::Error::Fatal("FDF: Can't create LDF\n");
    $ldf->init_layout($layout);

    $self->{'ldf'} = $ldf;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 print_html()

Print HTML

=cut
#------------------------------------------------------------------------------
sub print_html 
{
    my($self) = shift;

    print $self->{ldf}->print_html;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 add_row()

Adds a row to the HTML

=cut
#------------------------------------------------------------------------------
sub add_row 
{
    my($self) = shift;

    $self->{ldf}->add_row;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 no_rows_found()

Adds message to the HTML when no rows found.

=cut
#------------------------------------------------------------------------------
sub no_rows_found 
{
    my($self) = shift;

    $self->{ldf}->no_rows_found;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 verify( cgi )

Verifies all fields in the form, given the user input in cgi

Returns true(1) on success, false(0) on failure

=cut
#------------------------------------------------------------------------------
sub verify 
{
    my($self) = shift;
    my($cgi)  = shift;
    my($ret)  = 1;
    my($val); 
    my(@comp_fields);
    my($custom_verify,%record);

    $custom_verify = ( $self->{VERIFY} ? 1 : 0 );

    #----------------------------------------------------------
    # Get qualification either from the form or from the path  
    #----------------------------------------------------------
    if (  $cgi->param('wdb_keys') ) {
        print main::DEBUG "Query using keys\n" if $main::debug;
	@keys = split( '::', $cgi->param('wdb_keys') );
	foreach $key ( @{$self->{keys}} ) {
	    $f = $self->{field}{$key};
	    $f->set_value( shift @keys );
	    $f->set_oper( '==' );
            print main::DEBUG " - $key = $f->{value}\n" if $main::debug;
	}
    }

    foreach $f ( @{$self->{fields}} ) {
	#if ( defined $f->{computed} ) {	# Do computed fields last.
	#    push( @comp_fields, $f );
	#    next;
	#}
	if ( defined $cgi->param($f->{FIELD}) ) {
	    $val = $cgi->param($f->{FIELD});
	    $val =~ s/^\s*//;
	    $val =~ s/\s*$//;
	    if ( $f->verify( $val ) ) {
		$ret = 0;
	    }
	    #$f->set_value( $val ); # Done in verify...
	    $f->set_oper(  '=' );
	    $record{$f->{FIELD}} = $f->get_db_value() if $custom_verify;
	}
    }

#    #foreach $f ( @compfields ) {
#	if ( defined $cgi->param($f->{FIELD}) ) {
#	    $val = $cgi->param($f->{FIELD});
#	    $val =~ s/^\s*//;
#	    $val =~ s/\s*$//;
#	    if ( $f->verify( $val ) ) {
#		$ret = 0;
#	    }
#	    $f->set_oper(  '=' );
#	}
#    }

    if ( $custom_verify ) {
        print main::DEBUG "WDB::Fdf: Calling custom verify function: ".$self->{'VERIFY'}."\n" if $main::debug;
	#WDB::Eval::eval($fdf->{'NAME'}." VERIFY", $fdf->{'VERIFY'} );

	$err_list = eval ( $self->{'VERIFY'} . '(%record)' );
	WDB::Error::Fatal($self->{'NAME'}." VERIFY: $@") if $@;

	#$err_list = main::pfa_rec_verify(%record);

	if ($main::debug ) {
	    print main::DEBUG "----- FDF Custom verify returned -----\n" ;
	    print main::DEBUG "Num.errs : ". ($#err_list +1) ."\n" ;
	    print main::DEBUG "Err dump: ".join(',',@$err_list)."\n" ;
	    foreach $err ( @$err_list ) {
		foreach $key ( keys %$err ) {
		    print main::DEBUG $key." => ".$err->{$key} ."\n" ;
		}
	    }
	    print main::DEBUG "--------------------------------------\n" ;
	}

	foreach $err ( @$err_list ) {
	    $f = $self->{field}{$err->{FIELD}};
	    $f->{'errormsg'} .= $err->{errormsg};
	    $ret = 0;
            print main::DEBUG "WDB::Fdf: Custom verify : ".$err->{FIELD}.": ".$err->{'errormsg'}."\n" if $main::debug;
	}
    }

    return $ret;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 verify_query( cgi )

Verifies queries on all fields in the form, given the user input in cgi.
The operator and the query value is extracted and set in the field objects.
TODO: At the moment no verification is performed. 

Returns true(1) on success, false(0) on failure

=cut
#------------------------------------------------------------------------------
sub verify_query
{
    my($self) = shift;
    my($cgi)  = shift;
    my($ret)  = 1;
    my($val,@keys,$key,$f); 

    print main::DEBUG "FDF:verify_query()\n" if $main::debug;

    #----------------------------------------------------------
    # Get qualification either from the form or from the path
    #----------------------------------------------------------
    if (  $cgi->param('wdb_keys') ) {
        print main::DEBUG "Query using keys\n" if $main::debug;
	@keys = split( '::', $cgi->param('wdb_keys') );
	foreach $key ( @{$self->{keys}} ) {
	    $f = $self->{field}{$key};
	    $f->set_value( shift @keys );
	    $f->set_oper( '==' );
            print main::DEBUG " - $key = $f->{value}\n" if $main::debug;
	}
    }


    foreach $param ( $cgi->param ) {
    	print main::DEBUG "   :verifing $param\n" if $main::debug;

	next if ( ! defined ( $f = $self->{field}{$param}) );
	$query = $cgi->param($param);
	my ( $oper, $val ) = $query =~ /^\s*(<=|>=|!=|<|>|==|=|~)?(.*)$/;

    	print main::DEBUG "   : oper($oper), val($val)\n" if $main::debug;

	### Test
	    $val =~ s/^\s*//;
	    $val =~ s/\s*$//;
	    if ( $f->verify_query( $val ) ) {
		$ret = 0;
	    }
	    #$f->set_value( $val ); # Done in verify...
        #$f->set_value( $val );
        $f->set_oper( $oper );
    }

    return $ret;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 wdb_ACTION( %attr )

THIS IS A COPY OF THE FUNCION IN Ldf.pm !!!
TODO: Figure out a way to call wdb_ACTION from a Fdf, before the layout has
been initialized....

The ACTION tag has the following syntax :

    <WDB_ACTION NAME="action_name" FDF="..." LDF="...." KEYS>

    NAME	Name of the action to take (required)
    FDF		FDF to use (Optional - Default = current)
    LDF		LDF to use (Optional - Default given by action)
    KEYS	(Optional - Include uniqe keys to current row)
    QUERY	(Optional - Include all attributes of current query)

This tag is used in place of a URL.

=cut
#------------------------------------------------------------------------------
sub wdb_ACTION
{
    my($self)   = shift; 
    my(%attr)   = @_;
    my($url,$cgi,$key, @keys);
    my( $eparam,$value, @pairs,$querystring );

    #print main::DEBUG 'LDF: ACTION '.join(',',%attr)."\n" if $main::debug;

    #---------------------------------------------------------
    # Get the URL to the current script + fdf info PATH_INFO
    # TODO: (This is not that clean, and probably should be done
    # in a more generic way  ....)
    #---------------------------------------------------------
    $cgi = $self->{cgi};

    #$url = $cgi->url;
    # Error in CGI module on NT / Netscape Enterprise server ?
    $url = ( $cgi->script_name ) ? $cgi->script_name : "/cgi-bin/wdb";

    $url .= '/' . $attr{NAME};
    $url .= '/ldf=' . $attr{'LDF'}  if $attr{'LDF'};
    $url .= '/lastldf=' . $cgi->param('wdb_ldf');
    $url .= '/fdf=' . ( $attr{'FDF'} ? $attr{'FDF'} 
				: $cgi->param('wdb_fdf') );
    $url .= '/debug=1' if $main::debug;

    if ( $attr{KEEP} ) {
	foreach $keep ( split( ',', $attr{KEEP} ) ) {
	    my( $par, $val ) = split('=', $keep, 2 );
	    $url .= "/$par=" . ( $val ? $self->{field}{$val}->get_db_value()
				      : $cgi->param('wdb_'.$par) );
	}
    }

    if ( $attr{'QUERY'} ) {
	foreach ( $cgi->param ) {
	    unless ( /^wdb_(.*)$/ ) {
		my($eparam) = &main::cgi_encode($_);
		foreach $value ($cgi->param($_)) {
		    $value = &main::cgi_encode($value);
		    push(@pairs,"$eparam=$value");
		}
	    } 
	}
    }

    if ( $attr{KEYS} ) {
	#print main::DEBUG "LDF: ACTION adding KEYS\n" if $main::debug;
	foreach $key ( @{$self->{keys}} ) {
	   #print main::DEBUG "- KEY= $key\n" if $main::debug;
           push( @keys, &main::noblanks($self->{field}{$key}{'value'}) );
	}
        $url .= '/keys=' . join( '::', @keys );
    }

    if ( $attr{'QUERY'} ) {
	$querystring = join("&",@pairs);
	$url .= '?' . $querystring if ( $querystring );
    }

    #----------------------------------
    # Return URL for the given action.
    #----------------------------------
    
    return $url;
}

1;#			----- The End -----
