#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

Field_char - Definition of a 'char' FDF Field.

=head1 Description

This class defines a 'char' FDF field. Field_char is a subclass of Field

=head1 Public Methods
=cut#--------------------------------------------------------------------------

use WDB::Field;

package  WDB::Field_char; @ISA = qw( WDB::Field );
@WDB::Field_char::Inherit::ISA = @ISA;	# Access to overridden methods.
use English;   		                # Use long "english" variable names 


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 db_type()

Returns the database type of the field. 

=cut
#------------------------------------------------------------------------------
sub db_type
{
    my($self) = shift;

    return 'char'; 
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 default_oper()

Returns the default operand for queries

=cut
#------------------------------------------------------------------------------
sub default_oper
{
    my($self) = shift;

    if ( $self->{'enum'} ) {
	return '==';
    } else {
        return '~~';    # Case insensitive match anywhere in the string.
    }
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 html( %attr )

Return the HTML for the current field in the given layout. This functions
uses the following named parameters :

    TYPE => 	Layout type : [query|text|label|unit|toggle]
    DOTS =>	Append dots and a colon to the label.

=cut
#------------------------------------------------------------------------------
sub html
{
    my($self) = shift;
    my(%attr) = @_;
    my($html) = $self->SUPER::html(%attr);



    return $html;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 verify( value )

Verifies that the given value is valid for this field.

Returns an error message. An empty string indicates success.

Each field also keeps the result of the last verify in {'errormsg'}

=cut
#------------------------------------------------------------------------------
sub verify
{
    my($self)     = shift;
    my($val)      = shift;
    my($errormsg) = $self->SUPER::verify($val);

    if ( length($val) > $self->{'length'} ) {
        #$errormsg .= " Field too long - max length is $self->{'length'}";
	$errormsg .= sprintf($main::MSG{FIELD_TOO_LONG}, $self->{'length'});
    }

    $self->{'errormsg'} = $errormsg;
    return $errormsg;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 get_db_value

Returns the database value of the field.

=cut
#------------------------------------------------------------------------------
sub get_db_value
{
    my($self) = shift;
    my($val)  = $self->SUPER::get_db_value;

    $val =~ s/\'/\'\'/g;	# Escape singel quotes.

    return $val;
}


1;
