#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

Field_date - Definition of a 'date' FDF Field.

=head1 Description

This class defines a 'date' FDF field. Field_date is a subclass of Field


=head1 Public Methods
=cut#--------------------------------------------------------------------------

use WDB::Field;

package  WDB::Field_date; @ISA = qw( WDB::Field );
@WDB::Field_date::Inherit::ISA = @ISA;	# Access to overridden methods.
use English;   		                # Use long "english" variable names 

%month_name = (	'jan' =>  1,
		'feb' =>  2,
		'mar' =>  3,
		'apr' =>  4,
		'may' =>  5,
		'maj' =>  5,
		'jun' =>  6,
		'jul' =>  7,
		'aug' =>  8,
		'sep' =>  9,
		'oct' => 10,
		'okt' => 10,
		'nov' => 11,
		'dec' => 12 );

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 verify( value )

Verifies that the given value is valid for this field.

Returns an error message. An empty string indicates success.

Each field also keeps the result of the last verify in {'errormsg'}

=cut
#------------------------------------------------------------------------------
sub verify
{
    my($self)     = shift;
    my($val)      = shift;
    my($errormsg); # = $self->SUPER::verify($val);

    $self->{value} = $val;
    if ($val ne '' ) {
      $errormsg .= $self->check_date($val);
    }
    $errormsg .= $self->SUPER::verify($self->{value});
    $self->{'errormsg'} = $errormsg;
    return $errormsg;
}

sub verify_query
{
    my($self)     = shift;
    my($val)      = shift;
    my($errormsg); 

    $self->{value} = $val;
    if ($val ne '' ) {
      $errormsg .= $self->check_date($val);
    }
    $errormsg .= $self->SUPER::verify_query($self->{value});
    $self->{'errormsg'} = $errormsg;
    return $errormsg;
}


sub check_date
{
    my($self)     = shift;
    my($val)      = shift;
    my($errormsg); 

        my($day,$mon,$year);
	if ( $val =~ /^[0-9]+$/ ) {
	    $day  = substr($val, 0, 2);
	    $mon  = substr($val, 2, 2);
	    $year = substr($val, 4, 4);
	} else {
	    ($day,$mon,$year) = split ( /[-.\/\ ]+/, $val );
    	}
	
	$mon = $month_name{lc($mon)} if defined $month_name{lc($mon)};

	$errormsg .= " " unless ( $day =~ /^[0-9]+$/ );
	$errormsg .= " " unless ( $mon =~ /^[0-9]+$/ );
	$errormsg .= " " unless ( $year =~ /^[0-9]+$/ );

	if ( $day < 1 or $day > 31 ) {
	    #$errormsg .= " Day not between 1 and 31.";
	    $errormsg .= $main::MSG{ERR_DAY};
	}

	if ( $mon < 1 or $mon > 12 ) {
	    #$errormsg .= " Month not between 1 and 12.";
	    $errormsg .= $main::MSG{ERR_MONTH};
	}

	if ($year < 100 ) {
	    $year += ( $year < 60 ? 2000 : 1900 );
	}
	if ( $year > 9999 ) {
	    #$errormsg .= " Year: $year more than 9999.";
	    $errormsg .= $main::MSG{ERR_YEAR_TOOBIG};
	}

        # AOL 12/3-1998
	#$errormsg .= "The date $day-$mon does not exist." 
	$errormsg .= sprintf($main::MSG{ERR_DM_NOEXIST},$day,$mon)
           if (($day eq '31') && ($mon =~ /^(2|4|6|9|11|02|04|06|09)$/));  
	$errormsg .= sprintf($main::MSG{ERR_DM_NOEXIST},$day,$mon)
           if (($day eq '30') && ($mon =~ /^(2|02)$/));  
	$errormsg .= sprintf($main::MSG{ERR_DMY_NOEXIST},$day,$mon,$year)
           if (($day eq '29') && ($mon =~ /^(2|02)$/) && ($year%4) );  
	$errormsg .= sprintf($main::MSG{ERR_DMY_NOEXIST},$day,$mon,$year)
           if (($day eq '29') && ($mon =~ /^(2|02)$/) && (!($year%100)) && ($year%400) );  

	$self->{value} = sprintf "%02d-%02d-%4d", $day, $mon, $year  unless $errormsg;

	#$errormsg .= " Please enter date in the form DD-MM-YYYY" if $errormsg;
	$errormsg .= $main::MSG{DATEFORMAT} if $errormsg;
#    } else {
#	$self->{value} = $val;

    return $errormsg;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 db_type()

Returns the database type of the field. 

=cut
#------------------------------------------------------------------------------
sub db_type
{
    my($self) = shift;

    return 'datetime';
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 set_db_value( value )

Sets the value of the field given the database value. 
(Converts using from_db)

=cut
#------------------------------------------------------------------------------
#sub set_db_value
#{
#    my($self) = shift;
#    my($val)  = shift;
#
#    $self->{value} = $val;
#
#    if ( $self->{'from_db'} ) {
#	$val = WDB::Eval::eval($self->{FIELD}.':from_db', $self->{'from_db'});
#    }
#}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 get_db_value

Returns the database value of the field.
(Converts using to_db)

=cut
#------------------------------------------------------------------------------
sub get_db_value
{
    my($self) = shift;
    my($val)  = $self->SUPER::get_db_value;
    #my($val)  = $self->{value};

#    print main::DEBUG "get_db_value: $self->{FIELD}\n" if $main::debug;
#    if ( $self->{'to_db'} ) {
#	$val = WDB::Eval::eval($self->{FIELD}.':to_db', $self->{'to_db'});
#
#        print main::DEBUG "get_db_value: to_db $self->{'to_db'} = $val\n" if $main::debug;
#    }

    return 'NULL' if ( ! $val );

   #return "TO_DATE('$val', 'DD-MM-YYYY')";
    return $val;
}

