#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

Field_dkfloat - Definition of an 'dkfloat' FDF Field.
Same as float, but with danish notation ( , instead of . )

=head1 Description

This class defines a 'dkfloat' FDF field. Field_dkfloat is a subclass of Field

=head1 Public Methods
=cut#--------------------------------------------------------------------------

use WDB::Field;

package  WDB::Field_dkfloat; @ISA = qw( WDB::Field );
@WDB::Field_float::Inherit::ISA = @ISA;	# Access to overridden methods.
use English;   		                # Use long "english" variable names 


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 db_type()

Returns the database type of the field. 

=cut
#------------------------------------------------------------------------------
sub db_type
{
    my($self) = shift;

    return 'float';    
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 html( %attr )

Return the HTML for the current field in the given layout. This functions
uses the following named parameters :

    TYPE => 	Layout type : [query|text|label|unit|toggle]
    DOTS =>	Append dots and a colon to the label.

=cut
#------------------------------------------------------------------------------
sub html
{
    my($self) = shift;
    my(%attr) = @_;
    my($html) = $self->SUPER::html(%attr);

    return $html;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 verify( value )

Verifies that the given value is valid for this field.

Returns an error message. An empty string indicates success.

Each field also keeps the result of the last verify in {'errormsg'}

=cut
#------------------------------------------------------------------------------
sub verify
{
    my($self)     = shift;
    my($val)      = shift;
    my($errormsg) = $self->SUPER::verify($val);

#   if ( ($val =~ /[^0-9,+-]/) ) {
    if ( ($val =~ /^[+-]?[0-9]*\,?[0-9]*$/) ) {
    } else {
	#$errormsg .= " field is numeric!";
	$errormsg .= $main::MSG{FIELD_FLOAT};
    }

    $self->{'errormsg'} = $errormsg;
    return $errormsg;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 set_db_value( value )

Sets the value of the field given the database value. 
(Converts using from_db)

=cut
#------------------------------------------------------------------------------
sub set_db_value
{
    my($self) = shift;
    my($val)  = shift;

    $self->SUPER::set_db_value($val);

    $self->{value} =~ s/\./,/;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 get_db_value

Returns the database value of the field.
(Converts using to_db)

=cut
#------------------------------------------------------------------------------
sub get_db_value
{
    my($self) = shift;
    my($save) = $self->{value};
    $self->{value} =~ s/,/\./;
    my($val)  = $self->SUPER::get_db_value();
    $self->{value} = $save;
    return $val;
}

1;
