#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

Field_int - Definition of an 'int' FDF Field.

=head1 Description

This class defines a 'int' FDF field. Field_int is a subclass of Field

=head1 Public Methods
=cut#--------------------------------------------------------------------------

use WDB::Field;

package  WDB::Field_int; @ISA = qw( WDB::Field );
@WDB::Field_int::Inherit::ISA = @ISA;	# Access to overridden methods.
use English;   		                # Use long "english" variable names 


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 db_type()

Returns the database type of the field. 

=cut
#------------------------------------------------------------------------------
sub db_type
{
    my($self) = shift;

    return 'int';    
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 get_db_value

Returns the database value of the field.
(Converts using to_db)

=cut
#------------------------------------------------------------------------------
sub get_db_value
{
    my($self) = shift;
    my($val)  = $self->SUPER::get_db_value;

#    return 'NULL' if ( ! $val );
    return 'NULL' if ( $val eq '' );

    return $val;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 html( %attr )

Return the HTML for the current field in the given layout. This functions
uses the following named parameters :

    TYPE => 	Layout type : [query|text|label|unit|toggle]
    DOTS =>	Append dots and a colon to the label.

=cut
#------------------------------------------------------------------------------
sub html
{
    my($self) = shift;
    my(%attr) = @_;
    my($html) = $self->SUPER::html(%attr);



    return $html;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 verify( value )

Verifies that the given value is valid for this field.

Returns an error message. An empty string indicates success.

Each field also keeps the result of the last verify in {'errormsg'}

=cut
#------------------------------------------------------------------------------
sub verify
{
    my($self)     = shift;
    my($val)      = shift;
    my($errormsg) = $self->SUPER::verify($val);

#    if ( ($val =~ /[^0-9+-]/) ) {
    if ( ($val =~ /^[+-]?[0-9]*$/) ) {
    } else {
	$errormsg .= $main::MSG{FIELD_INTEGER};
    }

    $self->{'errormsg'} = $errormsg;
    return $errormsg;
}


1;

