#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

Field_url - Definition of a 'url' FDF Field.

=head1 Description

This class defines a 'url' FDF field. Field_url is a subclass of Field_char.
At the moment this only adds extra verification to the field - it has
to _look_ like a URL. 

- This could possibly be extended with a HTTP 'head' request to verify that 
the URL exists, but this is probably to restrictive....

=head1 Public Methods
=cut#--------------------------------------------------------------------------

use WDB::Field_char;

package  WDB::Field_url; @ISA = qw( WDB::Field_char );
@WDB::Field_url::Inherit::ISA = @ISA;	# Access to overridden methods.
use English;   		                # Use long "english" variable names 


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 verify( value )

Verifies that the given value is valid for this field.

Returns an error message. An empty string indicates success.

Each field also keeps the result of the last verify in {'errormsg'}

=cut
#------------------------------------------------------------------------------
sub verify
{
    my($self)     = shift;
    my($val)      = shift;
    my($errormsg) = $self->SUPER::verify($val);

    return unless $val;

    if ( $val !~ /^http:\/\// ) {
	$errormsg .= " This does not look like a URL. Expected http://...";
    }

    $self->{'errormsg'} = $errormsg;
    return $errormsg;
}

