#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 NAME

Fdf - Layout Definintion File for WDB. 

=head1 Description

This class defines different HTML layouts for the fields defined in the FDF.

=head1 Private Data

    $self->{fdf}
    $self->{layout}{...}
    $self->{header}     
    $self->{rowdef}     
    $self->{rowdef_args}
    $self->{footer}     
    $self->{html}       

=head1 Public Methods
=cut#--------------------------------------------------------------------------

package  WDB::Ldf;
use English;                    # Use long "english" variable names 


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 new( filename )

New creates a new Ldf object 

=cut
#------------------------------------------------------------------------------
sub new 
{
    my($type) = shift;
    my($self) = {};
    my($filename,$fdf) = @_;
    bless ($self, $type);

    if ( defined $filename ) {
	# TODO: Configuration parameters should be passed in a more clever way
	$self->read_file("$WDB::Eval::formdir/$filename");
    }
    $self->{fdf} = $fdf;

    $self->{http} = "Content-type: text/html\n\n"; 

    return $self;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 read_file( filename )

Reads the LDF file and stores the layouts in $self->{layout}{...}

=cut
#------------------------------------------------------------------------------
sub read_file
{
    my($self) = shift; 
    my( $file ) = @_;
    my($key, $void, $val );

    print main::DEBUG "LDF: Reading $file\n" if $main::debug;
    open( LDF, $file ) || WDB::Error::Fatal("Can't open $file: $!\n");

    $lineno = 0;
    while ( <LDF> ) {	  
        #print "LDF: Read $_\n" if $main::debug;
	$lineno++;

	#-----------------------------------------------------------------
	# Parse a line of the form : "key=value". Comments and unnessesary
	# blanks are skipped. (blanks in the middel of values are kept!)
	# The key should start at the 1. character on the line - lines 
	# starting with whitespace are interpreted as continuation lines.
	#-----------------------------------------------------------------
	next if /^#.*$/;
	($key, $void, $val ) = 
		/^(\w+)\s*(=\s*(.*))?$/;
		# /^(\w+)\s*(=\s*((\s*\".*\"|\s*[^#\s\n]+)+))?\s*(#.*)?$/;
		# /^(\w+)\s*(=\s*((\s*[^#\"\s\n]+|\s*\".*\")+))?\s*(#.*)?$/;

	if ( ! $key ) {   # Continuation line ...
            #print "LDF: (cont. of $old_ldf_key)\n" if $main::debug;
	    ($val) = /^\s*(.*)?$/;
	    next unless $val;
	    $self->{layout}{$old_ldf_key} .= "\n$val";
	    next;
	}
        #print main::DEBUG "LDF: Finnised Layout $old_ldf_key : \n$self->{layout}{$old_ldf_key}\n-----------\n" if $main::debug;

	$self->{layout}{$key} = $val;	
	$old_ldf_key = $key;

        #print main::DEBUG "LDF: Layout $key : \n$val\n-----------\n" if $main::debug;
    }
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 init_layout( layout_name )

Initializes the given layout

=cut
#------------------------------------------------------------------------------
sub init_layout
{
    my($self)   = shift; 
    my($layout_name) = @_;
    my(%rowdef_args);

    print main::DEBUG "LDF:Initializing layout $layout_name\n" if $main::debug;

    if ( ! defined $self->{layout}{$layout_name} ) {
	WDB::Error::Fatal("No such layout : $layout_name !");
    }

    my($layout) = $self->{layout}{$layout_name} ;
    #print main::DEBUG "LDF:---- $layout_name ----\n$layout\n" if $main::debug;
    #--------------------------------
    # Split Header / Rowdef / Footer
    #--------------------------------
    $self->{header}      = '';
    $self->{rowdef}      = '';
    $self->{rowdef_args} = {};
    $self->{footer}      = '';
    $self->{html}        = '';

    if ( $layout =~ /\<WDB_ROWDEF([^\>]*)\>/ ) {
	$self->{header} = $`;
	%rowdef_args = $self->_parse_attr($1);
	$self->{rowdef_args} = \%rowdef_args;
	$layout = $';
	if ( $layout =~ /\<\/WDB_ROWDEF([^\>]*)\>/ ) {
	    $self->{rowdef} = $`;
	    $self->{footer} = $';
	} else {
	    WDB::Error::Fatal("Error in LDF $layout_name: No </WDB_ROWDEF>\n");
	}
    } else {
	$self->{header} = $layout;
    }

    #print main::DEBUG "LDF:---- $layout_name header ----\n$self->{header}\n" if $main::debug;
    #print main::DEBUG "LDF:---- $layout_name rowdef ----\n$self->{rowdef}\n" if $main::debug;
    #print main::DEBUG "LDF:---- $layout_name footer ----\n$self->{footer}\n" if $main::debug;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 add_row

Adds a row to the HTML using the previously initialized layout.

=cut
#------------------------------------------------------------------------------
sub add_row
{
    my($self)   = shift; 
    #my(%row) = @_;

    if ( ! $self->{html} ) {
         print main::DEBUG "LDF: Add_Row: Parsing header first ...\n" if $main::debug;
	$self->parse( $self->{header} );
    }

    if ( $self->{rowdef} ) {
	$self->parse( $self->{rowdef} );
    }
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 no_rows_found

Adds message to the HTML when no rows found.

=cut
#------------------------------------------------------------------------------
sub no_rows_found
{
    my($self)   = shift; 

    if ( ! $self->{rowdef_args}{NONE} ) { 
	# NOP
	return;
    }

    if ( ! $self->{html} ) {
        ## ??? Maybe no_rows_found should produce the entire HTML page ?
	$self->parse( $self->{header} );
    }

    if ( $self->{layout}{$self->{rowdef_args}{NONE}} ) {
	$self->parse( $self->{layout}{$self->{rowdef_args}{NONE}} );
    } else {
	WDB::Error::Fatal("The layout $self->{rowdef_args}{NONE} is not defined!");
    }
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 print_html()

Returns the ready HTML page.

=cut
#------------------------------------------------------------------------------
sub print_html
{
    my($self)   = shift; 

    #print main::DEBUG "LDF:Print HTML\n" if $main::debug;

    #-----------------------------------------------
    # Print the header if this has not been done yet
    #-----------------------------------------------
    if ( ! $self->{html} ) {
	$self->parse( $self->{header} );
    }

    #-----------------------------------------------
    # Print the footer if it is there
    #-----------------------------------------------
    if ( $self->{footer} ) {
	$self->parse( $self->{footer} );
    }

    #-----------------------------------------------
    # Now return the HTML
    #-----------------------------------------------
    #print "LDF:HTML..................\n$self->{html}\n.............\n" if $main::debug;
    
    return $self->{http} . $self->{html};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 parse( LDF-code )

Parses a piece of LDF code and appends the result of the stored HTML.

=cut
#------------------------------------------------------------------------------
sub parse
{
    my($self)   = shift; 
    my($document) = @_;
    my(%attr,$key);

    while ( $document =~ /\<WDB_(\w*)?\s*([^\>]*)\>/ ) {
	$self->{html} .= $`;
	$tag = $1;
	%attr = $self->_parse_attr( $2 );
	$document = $';

	if ( $tag eq 'INC' ) {
	    if ( $attr{'SRC'} ) {
		open( INC, "$WDB::Eval::formdir/$attr{'SRC'}" ) || WDB::Error::Fatal("Can't open $attr{'SRC'}: $!\n");
		$document = join('',<INC>) . $document;
		close(INC);
	    } elsif ( $attr{'LAYOUT'} ) {
		$document = $self->{layout}{$attr{'LAYOUT'}} . $document;
	    } else {
		WDB::Error::Fatal("Error in WDB_INC tag: No SRC or LAYOUT attribute !\n");
	    }

	} elsif ($tag eq 'TEMPLATE') {
	    if ( $document =~ /\<\/WDB_TEMPLATE([^\>]*)\>/ ) {
		$template = $`;
		$document = $';

		$self->_parse_template( $template , %attr );

	    } else {
		WDB::Error::Fatal("Error in LDF $layout_name: No </WDB_TEMPLATE>\n");
	    }

	} else {
	    $self->{html} .= eval ('$self->wdb_'.$tag.'(%attr)' );
	    WDB::Error::Fatal("$@") if $@;
	}
    }
    $self->{html} .= $document;
    
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 _parse_template( string, %attr )

Parses a template LDF string.

=cut
#------------------------------------------------------------------------------
sub _parse_template
{
    my($self)   = shift; 
    my($template) = shift;
    my(%attr)	= @_;

    print main::DEBUG "_parse_template:--------\n$template\n----- " if $main::debug;
    foreach $f ( @{$self->{fdf}->{fields}} ) {
	if ( 
	     ($attr{ATTR} and $f->{$attr{ATTR}})
	     or
	     ($attr{NOATTR} and ! $f->{$attr{NOATTR}})
	     or
	     ( ! $attr{ATTR} and ! $attr{NOATTR})
	   ) {

		print main::DEBUG '- template field:'.$f->{FIELD}."\n" if $main::debug;
		$self->{default_NAME} = $f->{FIELD};
		$self->parse($template);
	}
    } 
}
    

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 parse_attr( string )

Parses the attributes to a <WDB_XXX ... > LDF tag, and returns the result
as an asociative array.

=cut
#------------------------------------------------------------------------------
sub _parse_attr
{
    my($self)   = shift; 
    my($string) = @_;
    my(%attr);

    while ( $string =~ /\s*(\w+)(="([^"]*)")?/ ) {
	$attr{$1} = ( $3 ? $3 : 1 );
	$string = $';
    }

    return %attr;
}
    

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 wdb_FIELD( %attr )

This asks the appropriate Field in the Fdf to produce its HTML. 
The FIELD tag has the following syntax :

    <WDB_FIELD NAME="field_name" TYPE="output_type">

- where I<field_name> is the name of the FDF field, and I<output_type>
is one of :

=over 4
=item query
=item text
=item label
=item unit
=item toggle
=back

=cut
#------------------------------------------------------------------------------
sub wdb_FIELD
{
    my($self)   = shift; 
    my(%attr)   = @_;

    #print main::DEBUG 'LDF: FIELD '.join(',',%attr)."\n" if $main::debug;
    #$self->{html} .= $self->{fdf}->{field}{$attr{NAME}}->html( $attr{TYPE} );

    $attr{NAME} = $self->{default_NAME} unless defined $attr{NAME} ;

    if ( ! $attr{NAME} ) {
	WDB::Error::Fatal("Error in <WDB_FIELD> tag - no NAME defined\n");
    }
    if ( ! $self->{fdf}->{field}{$attr{NAME}} ) {
	WDB::Error::Fatal("Error in <WDB_FIELD> tag - Field $attr{NAME} not defined\n");
    }

    $self->{fdf}->{field}{$attr{NAME}}->html( %attr );
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 wdb_PARAM( %attr )

This outputs the value of the given parameter.  The PARAM tag has the 
following syntax :

    <WDB_PARAM NAME="param_name">

- where I<param_name> is the name of the paramete 
Parameters are those special values typicaly pased in the PATH_INFO part 
of the URL to WDB.  

For example in 'http://www/cgi-bin/wdb/query/fdf=myfdf/mypar=myval' the
values of 'fdf' and 'mypar' kan be referenced with <WDB_PARAM NAME="fdf">
and <WDB_PARAM NAME="mypar">. Parameters kan also be pased as fields in
a form, but then the field name have to be 'wdb_' + the name of the
parameter.

=cut
#------------------------------------------------------------------------------
sub wdb_PARAM
{
    my($self) = shift; 
    my(%attr) = @_;
    my($cgi)  = $self->{fdf}->{cgi};

    if ( ! $attr{NAME} ) {
	WDB::Error::Fatal("Error in <WDB_PARAM> tag - no NAME defined\n");
    }

    print main::DEBUG "wdb_PARAM: $attr{NAME} =".$cgi->param('wdb_'.$attr{NAME})."\n" if $main::debug;

    $cgi->param('wdb_'.$attr{NAME});
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 wdb_VAR( %attr )

This outputs the value of the given variable.  The VAR tag has the 
following syntax :

    <WDB_VAR NAME="variable_name">

- where I<variable_name> is the name of the variable. Variables can be
either variabes set in the wdb.conf configuration file, or variables set
in on of the evaluated attributes in the FDF. F.eks. the PERL attr..

=cut
#------------------------------------------------------------------------------
sub wdb_VAR
{
    my($self) = shift; 
    my(%attr) = @_;
    my($cgi)  = $self->{fdf}->{cgi};

    if ( ! $attr{NAME} ) {
	WDB::Error::Fatal("Error in <WDB_VAR> tag - no NAME defined\n");
    }

    print main::DEBUG "wdb_VAR: $attr{NAME} =".$cgi->param('wdb_'.$attr{NAME})."\n" if $main::debug;

    WDB::Eval::eval("WDB_VAR:$attr{NAME}", '$'.$attr{NAME});
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 wdb_FDF( %attr )

This outputs the value of the FDF attribute.  The FDF tag has the 
following syntax :

    <WDB_FDF ATTR="fdf_attribute_name">

- where I<fdf_attribute_name> is the name of the FDF attribute. F.ex.
the TITLE or the NAME attributes

=cut
#------------------------------------------------------------------------------
sub wdb_FDF
{
    my($self) = shift; 
    my(%attr) = @_;
    my($ret);

    if ( ! $attr{ATTR} ) {
	WDB::Error::Fatal("Error in <WDB_FDF> tag - no ATTR defined\n");
    }

    $ret =  $self->{fdf}->{$attr{ATTR}};

    print main::DEBUG "wdb_FDF: $attr{ATTR} = $ret\n" if $main::debug;

    return ( $ret ? $ret : "" );
}



#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 wdb_ACTION( %attr )

The ACTION tag has the following syntax :

    <WDB_ACTION NAME="action_name" FDF="..." LDF="...." KEYS>

    NAME	Name of the action to take (required)
    FDF		FDF to use (Optional - Default = current)
    LDF		LDF to use (Optional - Default given by action)
    KEYS	(Optional - Include uniqe keys to current row)
    QUERY	(Optional - Include all attributes of current query)

This tag is used in place of a URL.

=cut
#------------------------------------------------------------------------------
sub wdb_ACTION
{
    my($self)   = shift; 
    my(%attr)   = @_;
    my($url,$cgi,$key, @keys);
    my( $eparam,$value, @pairs,$querystring );

    #print main::DEBUG 'LDF: ACTION '.join(',',%attr)."\n" if $main::debug;

    #---------------------------------------------------------
    # Get the URL to the current script + fdf info PATH_INFO
    # TODO: (This is not that clean, and probably should be done
    # in a more generic way  ....)
    #---------------------------------------------------------
    $cgi = $self->{fdf}->{cgi};

    #$url = $cgi->url;
    # Error in CGI module on NT / Netscape Enterprise server ?
    $url = ( $cgi->script_name ) ? $cgi->script_name : "/cgi-bin/wdb";

    $url .= '/' . $attr{NAME};
    $url .= '/ldf=' . $attr{'LDF'}  if $attr{'LDF'};
    $url .= '/lastldf=' . ( $attr{'LDF'} and $attr{'LDF'} eq $cgi->param('wdb_ldf') 
			    ? $cgi->param('wdb_lastldf')
			    : $cgi->param('wdb_ldf') );
    $url .= '/fdf=' . ( $attr{'FDF'} ? $attr{'FDF'} 
				: $cgi->param('wdb_fdf') );
    $url .= '/debug=1' if $main::debug;

    if ( $attr{KEEP} ) {
	foreach $keep ( split( ',', $attr{KEEP} ) ) {
	    my( $par, $val ) = split('=', $keep, 2 );
	    $url .= "/$par=" . ( $val ? $self->{fdf}{field}{$val}->get_db_value()
				      : $cgi->param('wdb_'.$par) );
	}
    }

    if ( $attr{SAVEQUERY} ) {
	# Encoding:  = => ! , & => | , % => @
	# TODO: Better (safe) encoding.
	my(@p);
	foreach ( $cgi->param ) {
	    unless ( /^wdb_(.*)$/ || /::/ ) {
		my($eparam) = &main::cgi_encode($_);
		foreach $value ($cgi->param($_)) {
		    $value = &main::cgi_encode($value);
		    $value =~ s/%/@/g;
		    push(@p,$eparam.'!'.$value);
		}
	    } 
	}
	$querystring = join('|',@p);
	$url .= '/query=' . $querystring if ( $querystring );
    }

    if ( $attr{QUERY} ) {
	foreach ( $cgi->param ) {
	    unless ( /^wdb_(.*)$/ ) {
		my($eparam) = &main::cgi_encode($_);
		foreach $value ($cgi->param($_)) {
		    $value = &main::cgi_encode($value);
		    push(@pairs,"$eparam=$value");
		}
	    } 
	}
    }

    if ( $attr{KEYS} ) {
	#print main::DEBUG "LDF: ACTION adding KEYS\n" if $main::debug;
	foreach $key ( @{$self->{fdf}->{keys}} ) {
	   #print main::DEBUG "- KEY= $key\n" if $main::debug;
           push( @keys, &main::noblanks($self->{fdf}{field}{$key}{'value'}) );
	}
        $url .= '/keys=' . join( '::', @keys );
    }

    if ( $attr{'QUERY'} ) {
	$querystring = join("&",@pairs);
	$url .= '?' . $querystring if ( $querystring );
    }
    if ( $attr{'LOADQUERY'} ) {
	$querystring = $cgi->param('wdb_query');
	$querystring =~ s/!/=/g;
	$querystring =~ s/\|/&/g;
	$querystring =~ s/@/%/g;
	$url .= '?' . $querystring if ( $querystring );
    }

    #----------------------------------
    # Return URL for the given action.
    #----------------------------------
    
    return $url;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 wdb_PREV( %attr )

The PREV tag generates a link in HTML to display the previous records in
the current query.

The PREV tag has the following syntax :

    <WDB_PREV ROWS="..." IMG="..." TXT="...">

    ROWS  =>	Number of rows to go back.
    IMG   =>	Image to use as anchor. (Optional)
    TXT   =>	Text to use as anchor.  (Optional)

At least on of the IMG or TXT attributes should be specified.

=cut
#------------------------------------------------------------------------------
sub wdb_PREV
{
    my($self)   = shift; 
    my(%attr)   = @_;
    my($html)	= '';
    my($cgi)	= $self->{fdf}->{cgi};
    my($start,$old_start);

    #--------------------------------------------------
    # Only display PREV if current start row > 1 
    #--------------------------------------------------
    if ( ($old_start = $cgi->param('wdb_start')) > 1 ) {

	#----------------------------------------------
	# Calculate new start row
	#----------------------------------------------
	$start = $old_start - $attr{ROWS};
	$start  = 0 if $start < 0;
	$cgi->param('wdb_start', $start );

	#----------------------------------------------
	# Produce self referencing URL tag.
	#----------------------------------------------
	my($url) = $cgi->self_url;
        # Error in CGI module on NT / Netscape Enterprise server ?
    	if ( $ENV{HTTPS} eq 'OFF' ) {
	    $url =~ s/https/http/;
	}

	my($txt) = ( $attr{TXT} ? $attr{TXT} : 'PREV' );
	$html = "<A HREF =\"$url\">";
	if ( $attr{IMG} ) {
	    $html .= '<IMG SRC="'.$attr{IMG}.'" BORDER=0>';
	} else {
	    $html .= $txt;
	}
	$html .= '</A>';

	$cgi->param('wdb_start', $old_start );
    }

    return $html;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 wdb_NEXT( %attr )

The NEXT tag generates a link in HTML to display the next records in
the current query.

The NEXT tag has the following syntax :

    <WDB_NEXT ROWS="..." IMG="..." TXT="...">

    ROWS  =>	Number of rows to go forward.
    IMG   =>	Image to use as anchor. (Optional)
    TXT   =>	Text to use as anchor.  (Optional)

At least on of the IMG or TXT attributes should be specified.

=cut
#------------------------------------------------------------------------------
sub wdb_NEXT
{
    my($self)   = shift; 
    my(%attr)   = @_;
    my($html)	= '';
    my($cgi)	= $self->{fdf}->{cgi};
    my($start,$old_start);


    #--------------------------------------------------
    # Only display NEXT if there are more rows...
    #--------------------------------------------------
    if ( $WDB::rows < 0 ) {

	#----------------------------------------------
	# Calculate new start row
	#----------------------------------------------
	$old_start = $cgi->param('wdb_start');
	$start     = $old_start + $attr{ROWS};
	$cgi->param('wdb_start', $start );

	#----------------------------------------------
	# Produce self referencing URL tag.
	#----------------------------------------------
	my($url) = $cgi->self_url;
        # Error in CGI module on NT / Netscape Enterprise server ?
    	if ( $ENV{HTTPS} eq 'OFF' ) {
	    $url =~ s/https/http/;
	}

	my($txt) = ( $attr{TXT} ? $attr{TXT} : 'NEXT' );
	$html = "<A HREF =\"$url\">";
	if ( $attr{IMG} ) {
	    $html .= '<IMG SRC="'.$attr{IMG}.'" BORDER=0>';
	} else {
	    $html .= $txt;
	}
	$html .= '</A>';
	$cgi->param('wdb_start', $old_start );
    }


    return $html;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 wdb_HTTP( %attr )

This sets the value of the HTTP header. Default is "Content-type: text/html\n\n".

    <WDB_HTTP HEADER="...">


=cut
#------------------------------------------------------------------------------
sub wdb_HTTP
{
    my($self) = shift; 
    my(%attr) = @_;
    my($cgi)  = $self->{fdf}->{cgi};

    if ( ! $attr{HEADER} ) {
	WDB::Error::Fatal("Error in <WDB_HTTP> tag - no HEADER defined\n");
    }

    print main::DEBUG "wdb_HTTP: HEADER =".$attr{HEADER}."\n" if $main::debug;

    $attr{HEADER} =~ s/\\n/\n/g;
    $self->{http} = $attr{HEADER};

    return '';
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 Author(s)

Bo Frese Rasmussen, DTV  9-Sep-1996

=head1 Revision Log

$Log: Ldf.pm,v $
Revision 1.7  2000/02/20 12:21:47  bfr
First version of template support in LDF files

Revision 1.6  2000/01/06 18:56:26  bfr
*** empty log message ***

Revision 1.5  1999/11/16 22:11:38  bfr
Klar til PFA lev. 17/11-1999

Revision 1.4  1999/03/30 17:19:51  bfr
no message

Revision 1.3  1999/03/29 22:42:49  bfr
no message

Revision 1.2  1998/09/17 10:06:22  bfr
Use lastldf

Revision 1.1.1.1  1998/08/27 17:16:58  bfr
wdb imported

# Revision 1.11  1997/11/07  13:48:37  aol
# *** empty log message ***
#
# Revision 1.10  1997/05/05  15:20:45  bfr
# *** empty log message ***
#
# Revision 1.9  1997/04/15  11:47:16  bfr
# Added Error handling (WDB::Error)
#
# Revision 1.8  1997/02/26  13:54:16  bfr
# Added HTTP header control and fixed problem with NULL values.
#
# Revision 1.7  1997/02/23  11:39:08  bfr
# First Public Release of Phonebook
#
# Revision 1.6  1997/02/22  13:20:54  bfr
# Ready for release.
#
# Revision 1.5  1997/02/18  09:43:01  bfr
# Checkpoint
#
# Revision 1.4  1997/02/04  16:47:08  bfr
# *** empty log message ***
#
# Revision 1.3  1996/12/20  17:10:42  bfr
# Added persistent wdb_* parameters in ACTION
#
# Revision 1.2  1996/10/31  11:18:26  bfr
# Ready for release 1.6.0
#
# Revision 1.1  1996/09/26  16:35:33  bfr
# First attempted at a new version with OO structure, Autoload, LDF's etc.
# This is not a stable version !
#
=cut#--------------------------------------------------------------------------

1; #			----- The End -----
