#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 action_release

The 'release' action for WDB.

=head1 Description

'Releases' a row in the database. This is an action developed specificaly for
the DARWIN project. It requires the FDF to define two extra form attributes.

RELEASE_TABLE = name of table containing released data.
RELEASE_UPDATE= update action to perform on the row in the 'working' table 
		to indicate that the data has been released. If left empty 
		or undefined no update is performed.

This module is automatically loaded by WDB when the 'release' action is
used.

=head1 Functions
=cut#--------------------------------------------------------------------------


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 action_release

Releases a row

=cut
#------------------------------------------------------------------------------
sub action_release
{
    my($cgi,$fdf) = @_;
    my($dbi,@keys,$key);
    my($val);

    print main::DEBUG "*** action_release ***\n" if $main::debug;
    WDB::Error::Fatal("No RELEASE_TABLE defined in form.") unless $fdf->{RELEASE_TABLE};

    #------------------------
    # VERIFY
    #------------------------
    if ( ! $fdf->verify($cgi) ) {
	WDB::Error::Fatal("Unexpected error. Verify failed in action_release.");
    } else {
	$WDB::Eval::ERRORMSG = '';
    }

    #------------------------
    # LOAD DBI
    #------------------------
    #$dbi = load_dbi($fdf->{'DATABASE'});
    $dbi = load_dbi();

    #------------------------------------
    # COMPOSE SQL AND RELEASE TABLE
    #------------------------------------
    my($table) = $fdf->{'TABLE'} =~ /([^,]+).*/; 

    foreach $f ( @{$fdf->{fields}} ) {
	#------------------------
	# Key field ?
	#------------------------
        if ( defined $f->{'key'} ) {
	    print main::DEBUG "Release: $f->{FIELD} - key" if $main::debug;

	    $val = $f->get_db_value();
            if ( ! defined $val || $val eq '' ) {
                WDB::Error::Fatal("Fatal error: missing key data.");
            } else {
		my($col) = $f->{column} =~ /.*?\.?([^\.]+)$/;
	        print main::DEBUG "($col)\n" if $debug;
		push( @where, eval('$dbi->query_' . $f->db_type 
			       . '($col,$f->{oper},$f->{value})') );
            }

        }
    }

    my($where) = join(' AND ',@where);
    do_release( $table, $fdf->{RELEASE_TABLE}, $where, $fdf->{RELEASE_UPDATE},0, $fdf->{RELEASE_SUB} ); 

    #----------------------------------
    # PRINT RESULT.
    #----------------------------------

    $fdf->layout( ( $cgi->param('wdb_ldf') ? $cgi->param('wdb_ldf') : 'release') );
    $fdf->print_html;
    #$dbi->close();
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 do_release

Releases one row from a table, by copying it to another similar
table, and optionally update it's status in the original table.

arguments:
    $from_table		Table to copy data from
    $to_table		Table to copy data to (released data)
    $where		SQL 'where' clause identifying data in the 'from' table
    $update		'update' clause to change status of 'from' table.
    $keys		Key fields in the 'from' table. (used in update)
    $release_sub 	Special structure defining sub-tables to release.
=cut
#------------------------------------------------------------------------------
sub do_release
{
    my($from_table, $to_table, $where, $update, $keys, $release_sub ) = @_;
    my($sql_stmt,%row);
    my($dbi) = load_dbi();
    my(@keys) = split(':', $keys);

    print main::DEBUG "\n********** do_release **********\n"
		    . "from_table.: $from_table\n"
		    . "to_table...: $to_table\n"
		    . "where......: $where\n"
		    . "update.....: $update\n"
		    . "keys.......: $keys\n"
		    . "release_sub: $release_sub\n"
		    if $main::debug;

    # TEST THIS!
    $dbi->open("delete from $to_table where $where");

    $dbi->open("select * from $from_table where $where");

    while ( %row = $dbi->nextrow ) {
        my($subdbi) = load_dbi();

	#---------------------------------------------------
        # If keys are defined, construct new where clause.
	#---------------------------------------------------
	if ($keys) {
	    my(@kw,$key);
	    foreach $key ( @keys ) {
		push(@kw, $key . "='$row{$key}'" );
	    }
	    $where = join(' AND ',@kw);
	}

	#------------------------
        # First try to update.
	#------------------------
	my(@upds);
	foreach $col ( keys %row ) {
	    $row{$col} =~ s/'/''/g;
	    push(@upds, $col . "='$row{$col}'" );
	}
	$sql_stmt = "update $to_table set " . join(',',@upds)
		  . ' where ' . $where;

        print main::DEBUG "Try: $sql_stmt\n" if $main::debug;
	my($ret) = $subdbi->try($sql_stmt);
        print main::DEBUG "   : return=$ret ($WDB::DBI_Oracle::ora_errstr)\n" if $main::debug;
	if ( $ret < 1 ) {
	    #--------------------------------
	    # If update fails, try insert...
	    #--------------------------------
	    my(@cols,@vals,$col);
	    foreach $col ( keys %row ) {
		push(@cols,$col);
		push(@vals,$row{$col});
	    }
	    $sql_stmt = "insert into $to_table ("
		      . join(',', @cols ) . ')'
		      . 'values (\'' . join('\',\'', @vals) . '\')';

	    print main::DEBUG "Do: $sql_stmt\n" if $main::debug;
	    $subdbi->do($sql_stmt);
	}

	#-------------------------------------
        # Update 'from_table' if needed.
	#-------------------------------------
	if ($update) {
	    print main::DEBUG "Do: update $from_table set $update where $where\n" if $main::debug;
	    $subdbi->do("update $from_table set $update where $where");
	}

	#----------------------------------
	# RELEASE SUB-TABLES (IF ANY)
	#----------------------------------
	if ( $release_sub ) {
	    my($sub);
	    foreach $sub ( split( ';', $release_sub ) ) {
		my($from_table, $to_table, $where, $update, $keys ) = split(',',$sub);
		#-------------------------
		# Modify where clause....
		#-------------------------
	        print main::DEBUG "Modifying where: $where\n" if $main::debug;
		while ( $where =~ /\$(\w+)/ ) {
		    $new_where = $` . $row{$1};
	            print main::DEBUG "  $1 = $row{$1}\n" if $main::debug;
		    $where = $';
		}
		$where = $new_where . $where;

		#-------------------------
		# Release sub-table data
		#-------------------------
		do_release( $from_table, $to_table, $where, $update, $keys ); 
	    }
	}

	#$subdbi->close();
    } 
    #else {
    #	WDB::Error::Fatal("Couldn't find data to release in $from table where $where ?");
    #}
    #$dbi->close;
}

1;
