#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head1 action_upd_list

The 'upd_list' action for WDB.

=head1 Description

Updates a row in the database. Called from the form generated by the 'upd_form'
action. (The <WDB_FIELD> tags should use the KEYS argument)

This module is automatically loaded by WDB when the 'upd_list' action is
used.

=head1 Functions
=cut#--------------------------------------------------------------------------


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 action_upd_list

Updates the database.

=cut
#------------------------------------------------------------------------------
sub action_upd_list
{
    my($cgi,$fdf) = @_;
    my($dbi,@keys,$key,$keylist);
    my($val);
    my(%param,%h,$f,$p,@key_order);
    my($verify_ok);
    my($display_errors) = 0;
    my($custom_verify) = ( $fdf->{VERIFY} ? 1 : 0 );

    print main::DEBUG "*** action_upd_list ***\n" if $main::debug;

    #------------------------
    # LOAD DBI
    #------------------------
    $dbi = load_dbi($fdf->{'DATABASE'});

    #------------------------
    # SORT INPUT PARAMETERS
    #------------------------
    foreach $p ( $cgi->param() ) {
	( $f, $keylist ) = split( '::', $p, 2 );
	if ( $f =~ /wdb_.*/ ) {
	    if ( $f eq 'wdb_ustart' ) {
		my( $start, $ldf ) = split( '\.', $keylist );
		$cgi->param('wdb_start',$start);
		$cgi->param('wdb_ldf',$ldf);
		print main::DEBUG "PARAM: $p : Set LDF= $ldf , START=$start \n" if $main::debug;
	    }
	    print main::DEBUG "PARAM: skiped $f = ".$cgi->param($p)."\n" if $main::debug;
	} else {
	    if ( ! defined $param{$keylist} ) {
		# Remember order of keys (not guaranteed to work according to CGI standard?)
		push( @key_order, $keylist );
	        print main::DEBUG "NEW KEY : $keylist\n" if $main::debug;
	    }
	    $param{$keylist}{$f} = $cgi->param($p);
	    print main::DEBUG "PARAM: $keylist, $f = $param{$keylist}{$f} \n" if $main::debug;
	}
    }

    #------------------------------------------
    # FOR EACH SET OF KEYS - UPDATE THE FIELDS
    #------------------------------------------
    #foreach $keylist ( keys %param ) {
    foreach $keylist ( @key_order ) {
	my(%h);
	my(%record);
	print main::DEBUG "Key-list: $keylist \n" if $main::debug;
	#----------------------------------------------------------
	# Set key fields
	#----------------------------------------------------------
	@keys = split( '::', $keylist);
	foreach $key ( @{$fdf->{keys}} ) {
	    $f = $fdf->{field}{$key};
	    $f->set_value( shift @keys );
	    $f->set_oper( '==' );
	    print main::DEBUG " - $key = $f->{value}\n" if $main::debug;
	}

	#----------------------------------------------------------
	# Verify and Prepare form fields.
	#----------------------------------------------------------
	$verify_ok = 1;
	%h = %{$param{$keylist}};
	foreach $f ( @{$fdf->{fields}} ) {
	    if ( defined $h{$f->{FIELD}} ) {
		$val = $h{$f->{FIELD}};
		$val =~ s/^\s*//;
		$val =~ s/\s*$//;
		if ( $f->verify( $val ) ) {
		    $verify_ok = 0;
		}
		#$f->set_value( $val ); # Done in verify...
		$f->set_oper(  '=' );
	        $record{$f->{FIELD}} = $f->get_db_value() if $custom_verify;
	    }
	}

	#----------------------------------------------------------
	# Custom Verify ?
	#----------------------------------------------------------

	if ( $custom_verify ) {
	    print main::DEBUG "action_upd_list Calling custom verify function: ".$fdf->{'VERIFY'}."\n" if $main::debug;

	    $err_list = eval ( $fdf->{'VERIFY'} . '(%record)' );
	    WDB::Error::Fatal($fdf->{'NAME'}." VERIFY: $@") if $@;

	    if ($main::debug ) {
		print main::DEBUG "----- upd_list Custom verify returned -----\n" ;
		print main::DEBUG "Num.errs : ". ($#err_list +1) ."\n" ;
		print main::DEBUG "Err dump: ".join(',',@$err_list)."\n" ;
		foreach $err ( @$err_list ) {
		    foreach $key ( keys %$err ) {
			print main::DEBUG $key." => ".$err->{$key} ."\n" ;
		    }
		}
		print main::DEBUG "--------------------------------------\n" ;
	    }

	    foreach $err ( @$err_list ) {
		$f = $fdf->{field}{$err->{FIELD}};
		$f->{'errormsg'} .= $err->{errormsg};
		$verify_ok = 0;
		print main::DEBUG "WDB::Fdf: Custom verify : ".$err->{FIELD}.": ".$err->{'errormsg'}."\n" if $main::debug;
	    }
	}

	#----------------------------------------------------------
	#----------------------------------------------------------
	#----------------------------------------------------------
#	$cgi->param('wdb_keys', $keylist);
#	%h = %{$param{$keylist}};
#	foreach $f ( keys %h ) {
#	    $cgi->param($f,$h{$f});
#	    print main::DEBUG " - $f = $h{$f}\n" if $main::debug;
#	}

#	foreach $f ( @{$fdf->{fields}} ) {
#	    if ( defined $h{$f->{FIELD}} ) {
#		$cgi->param($f->{FIELD},$h{$f->{FIELD}});
#	    }
#	}

	#----------------------------------------------------------
	# Update entry
	#----------------------------------------------------------
	#if ( $fdf->verify($cgi) ) {
	if ( $verify_ok ) {
	    $dbi->update( $cgi, $fdf );
	} else {
	    #--------------------------------------------------------------
	    # Load the current layout again if we haven't done this before.
	    #--------------------------------------------------------------
	    if ( ! $display_errors ) {
		$display_errors = 1;
		#$fdf->layout( ( $cgi->param('wdb_currentldf') ? $cgi->param('wdb_currentldf') : 'list_tab') );
		$fdf->layout( ( $cgi->param('wdb_lastldf') ? $cgi->param('wdb_lastldf') : 'list_tab') );
	    }

	    #-----------
	    # Set keys
	    #-----------
#	    @keys = split( /::/, $keylist);
#	    foreach $key ( @{$fdf->{keys}} ) {
#	       $in{$key} = shift(@keys);
#	    }
	    foreach $f ( @{$fdf->{fields}} ) {
		if ( defined $f->{computed} ) {	
		    $f->set_db_value();
		}
	    }

	    #-----------------------------------------------
	    # Add a row to the output with the current data.
	    #-----------------------------------------------
	    $fdf->add_row;
	}
    }

    if ( $display_errors ) {
	# ?
    } else {
	$fdf->layout( ( $cgi->param('wdb_ldf') ? $cgi->param('wdb_ldf') : 'liste') );
    }
    
    $fdf->print_html;
    return;
}

1;
