#!/usr/local/bin/perl
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.IDENTIFICATION  wdb
#.LANGUAGE	  Perl5 script
#
#.PURPOSE	  WWW / Database Interface.
#
#.AUTHOR	  Bo Frese Rasmussen <bfr@dtv.dk>
#
#.VERSION	1.0	14/06-1994  Creation
#.VERSION	1.1	11/10-1994  The following bugs where fixed :
#				    Query on 0 was ignored.
#				    Range query (..) f...ed up subsequent
#				    queries in the same form.
#				    Range queries on datatime fields didn't
#				    work.
#					
#				    Dateformat config. option added.
#
#.VERSION	1.2	30/12-1994  Support for spec. null values added.
#				    'add_menu' function to allow extra menu
#				    options to be added from fdf.
#				    Fixed bug with continuation lines for
#				    field attributes.
#					
#.VERSION	1.3a1	03/03-1995  'sameline' field attribute
#				    added.
#				    'html' field attribute added.
#				    Perl5 compatible.
#				    Query on computed fields. 
#				    Uses single quotes rather than
#				    double quotes around strings.
#
#.VERSION	1.4	05/03-1996  Minor clean-ups....
#
#.VERSION	1.5	12/06-1996  Insert/Update/Delete functionality added
#				    based on Jeff Rowes (j.p.rowe@larc.nasa.gov)
#				    WDB enhancements tutorial
#				    http://cscsun1.larc.nasa.gov/~beowulf/tutor/
#
#.VERSION	1.6	06/09-1996  Split in autoloaded modules, added LDF's
#
#.VERSION	1.7	22/02-1997  Many more enhancements. Ready for production
#				    system.
#					
#.COPYRIGHT (see end of file)
#------------------------------------------------------------------------------
$version       = "wdb 1.7";
$rel_date      = "22-feb-97";
$short_version = "w1.7";

#-------------------------------------------------------
# Get CGI stuff
#-------------------------------------------------------
#-------------------------------------------------------
# Get CGI stuff
#-------------------------------------------------------
#BEGIN { 
#    #(($CONFIG_DIR) = ($_=readlink($0))?$_=~m|(.*)/[^/]+$|:$0=~m|(.*)/[^/]+$|)?$_=1:$root='.';
#    $CONFIG_DIR = 'C:/wdb';
#    require "c:/kp.pl";
#    $CONFIG_DIR = $WDB_DIR;
#    $LIB_DIR = $CONFIG_DIR ; 
#    unshift( @INC, $CONFIG_DIR) 
#};
#


BEGIN { 
    #(($CONFIG_DIR) = ($_=readlink($0))?$_=~m|(.*)/[^/]+$|:$0=~m|(.*)/[^/]+$|)?$_=1:$root='.';
    #$CONFIG_DIR = 'e:/wdb';
    if ( $^O eq 'MSWin32' ) {
	#require "c:/kp.pl";
	#$LIB_DIR = $CONFIG_DIR ; 
	$LIB_DIR = 'E:/pfa/wdb';
	$CONFIG_DIR = 'c:/wdb';
    } else {
	(($LIB_DIR) = ($_=readlink($0))?$_=~m|(.*)/[^/]+$|:$0=~m|(.*)/[^/]+$|)?$_=1:$root='.';
	$CONFIG_DIR = '/etc/wdb';
    }
    #$CONFIG_DIR = $WDB_DIR;
    unshift( @INC, $LIB_DIR);
};
require WDB::Error;
require WDB::Fdf;
require WDB::Autoload;
require WDB::Eval;
use CGI qw(:cgi-lib);
#use CGI qw(:standard);
use CGI::Carp qw/fatalsToBrowser/;


$cgi = &load_cgi();
$WDB = ( $cgi->script_name ) ? $cgi->script_name : "/cgi-bin/wdb";

$WDB::Eval::WDB = $WDB;

#-------------------------------------------------------
# Prepare STDOUT and STDERR
#-------------------------------------------------------
#open(STDERR,">&STDOUT") || WDB::Error::Fatal("Can't dup stdout: $!\n");
#select(STDERR); $| = 1; # Make unbuffered.
select(STDOUT); $| = 1; # Make unbuffered.

#print "Content-type: text/html\n\n"; 
#print $cgi->header;

#-------------------------------------------------------
# Get local configuration parameters.
#-------------------------------------------------------
$LINEWIDTH = 65;

#($CONF_NAME) = $0 =~ m.*?([^/]+)$;
($CONF_NAME) = $WDB =~ m.*?([^/.]+)\.?([^.]*)$;

if ( -f "$CONFIG_DIR/$CONF_NAME.conf" ) {
    $CONF_FILE = "$CONFIG_DIR/$CONF_NAME.conf";
} elsif ( -f "$CONFIG_DIR/wdb.conf" ) {
    $CONF_FILE = "$CONFIG_DIR/wdb.conf";
} else {
    WDB::Error::Fatal("Configuration file '$CONFIG_DIR/wdb.conf' not found ?\n");
}

WDB::Eval::eval("Error reading configuration file $CONF_FILE",
		  "do '$CONF_FILE'");

WDB::Error::Logfile(  $WDB::Eval::ERROR_LOGFILE ) if $WDB::Eval::ERROR_LOGFILE ;
WDB::Error::MailAddr( $WDB::Eval::ERROR_EMAIL   ) if $WDB::Eval::ERROR_EMAIL ;

# Load language specific message file
if (! do 'Msg_'.$WDB::Eval::LANG.'.pm' ) {
  do 'Msg_ENG.pm' || WDB::Error::Fatal("Error loading message file Msg_ENG.pm\n$!\n");
}
#require 'Msg_ENG.pm'


#-------------------------------------------------------------------------
# DEBUG stuff...
# If your are making changes to the wdb script you might find it usefull
# to have a test/development version and an installed version. Set the
# variable $debug in your configuration file.
# This will enable some extra debug output to be printed in the HTML
# pages and in a debug file. If you are making changes to wdb, please
# let me know.
#-------------------------------------------------------------------------
$debug=$cgi->param('wdb_debug');;
if ( $debug ) {
#    if ( $cgi->param('wdb_action') eq "form" ) { 
#	open(DEBUG,"> $WDB::Eval::debugfile");
#	#printf DEBUG "*** $version : $dbdir / $formname ".`date`." ***\n\n";
#	printf DEBUG "*** $version : $dbdir / $formname  ***\n\n";
#    } else {
	open(DEBUG,">> $WDB::Eval::debugfile");
#    }
    printf DEBUG "\n\n*********************************************************************\n";
    printf DEBUG     "**** Action : ".$cgi->param('wdb_action') . "\n";
    printf DEBUG     "**** Fdf    : ".$cgi->param('wdb_fdf')    . "\n";
    printf DEBUG     "**** Ldf    : ".$cgi->param('wdb_ldf')    . "\n";
    printf DEBUG     "**** Config : $CONF_NAME\n";
    printf DEBUG     "*********************************************************************\n";
    select(DEBUG); $| = 1; # Make unbuffered.
    select(STDOUT);

    print DEBUG "\n**** Environment :\n";
    foreach $env ( sort keys(%ENV) ) {
	printf DEBUG "$env = $ENV{$env}\n";
    }

    print DEBUG "\n**** CGI :\n";
    foreach $key ( keys ( %in ) ) {
	print DEBUG "$key  = $in{$key} \n";
    }
    print DEBUG "\n****\n";
}

#---------------------
# Main() ...
#---------------------

if ( $INIT_FUNC ) {
    eval ($INIT_FUNC . '($cgi)' );
    WDB::Error::Fatal("$@") if $@;
}

$fdf = new WDB::Fdf($WDB::Eval::formdir,$cgi);
WDB::Eval::init($fdf);

eval ('action_' . $cgi->param('wdb_action') . '($cgi, $fdf)' );
WDB::Error::Fatal("$@") if $@;

printf DEBUG "\n\n**** WDB Exited normally ... ****************************************\n" if $debug;
exit 0;


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Convert SGML special Character codings into ASCII
#
#.REMARKS
#
#.RETURNS       The converted string
#------------------------------------------------------------------------------

sub SGMLtoASCII
{
    local ($str) = @_;
    $str =~ s/&gt;/>/g;
    $str =~ s/&lt;/</g;
    return( $str );
}

#
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE	Encodes a string so it doesn't cause problems in URL.
#
#.REMARKS
#
#.RETURNS	The encoded string
#------------------------------------------------------------------------------

sub cgi_encode
{
    local ($str) = @_;
    $str = &escape($str,'[\x00-\x20"#%/+;<>?\x7F-\xFF]');
    $str =~ s/ /+/g;
    return( $str );
}


# ===========================================================================
# escape(): Return the passed string after replacing all characters matching
#           the passed pattern with their %XX hex escape chars.  Note that
#           the caller must be sure not to escape reserved URL characters
#           (e.g. / in pathnames, ':' between address and port, etc.) and thus
#           this routine can only be applied to each URL part separately. E.g.
#
#           $escname = &escape($name,'[\x00-\x20"#%/;<>?\x7F-\xFF]');
#
sub escape
{
    local($str, $pat) = @_;
         
    $str =~ s/($pat)/sprintf("%%%02lx",unpack('C',$1))/ge;
    return($str);
}

# ===========================================================================
# unescape(): Return the passed URL after replacing all %NN escaped chars
#             with their actual character equivalents.
#
sub unescape
{
    local($url) = @_;

    $url =~ s/%(..)/pack("C",hex($1))/ge;
    return $url;
}


sub e_escape
{
    local($str, $pat) = @_;
    $pat ='[\x00-\x20"#%/+;_!<>?\x7F-\xFF]' unless $pat;
         
    $str =~ s/($pat)/sprintf("_%02lx",unpack('C',$1))/ge;
    return($str);
}

sub e_unescape
{
    local($url) = @_;

    $url =~ s/_(..)/pack("C",hex($1))/ge;
    return $url;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 load_dbi

Loads the database interface. - Dies on failure !

=cut
#------------------------------------------------------------------------------
sub load_dbi
{
    my($database) = @_;
    my($dbi);
    
    #--------------------------
    # Load database interface.
    #--------------------------
    require 'WDB/DBI_'.$WDB::Eval::DBI.'.pm'
	|| WDB::Error::Fatal("Error loading database interface file WDB/DBI_$WDB::Eval::DBI.pm :\n$!\n");
    
    $dbi = eval("new WDB::DBI_$WDB::Eval::DBI(\$WDB::Eval::user, \$WDB::Eval::pswd, \$WDB::Eval::server, \$WDB::Eval::database)");
    if ( $@ ) { WDB::Error::Fatal("Load_DBI failed: $@\n") }

    $dbi->dateformat($WDB::Eval::DATEFORMAT) if $WDB::Eval::DATEFORMAT;

    return $dbi;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 load_cgi

Loads the CGI info, and returns a CGI object. 
Also parses the PATH_INFO with the following syntax :
    /<action>/<param>=<val>/<param>=<val>/...

The <action> is the action to take. ( form/query/delete/update/... ).
The <param> parameters are inserted as CGI parameters in the  CGI
object with 'wdb_' prepended to the parameter name. This allows for 
these parameters to be passed either in PATH_INFO, or as fields in a form.

The parameters depend on the action, but are typically :

    fdf	  =>	The name of the FDF to use. Colons ':' are replaced with
		slashes ('/') before the file is opened, to allow for
		the FDF's to be arrangend in sub-dir's.
    ldf   =>	The LDF to use. ( - Most actions have a default. ) 
    start =>	Start row. (For next / previous)
    rows  =>	Max number of rows to return (for next /previous )


TODO: At the moment this also initializes the %in array for backwards 
compatibility - this should eventually be removed.

=cut
#------------------------------------------------------------------------------
sub load_cgi
{
    my( @path, $param, $key, $val, $cgi );

    &ReadParse;
    #$cgi = new CGI;
    $cgi = $in{CGI};

    @path = split( '/', $cgi->path_info );
    shift(@path);	# - There is nothing before the first '/'

    $cgi->param('wdb_action', shift(@path) );

    foreach $param ( @path ) {
	($key,$val) = split( '=', $param );
	$cgi->param('wdb_'.$key, $val ) unless $cgi->param('wdb_'.$key);
    }

    return $cgi;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
=head2 wdb_url

Returns a URL to with the same parameters as the current WDB session.

=cut
#------------------------------------------------------------------------------
sub wdb_url
{
    my( $cgi ) = @_;
    my( $url );

    $url = $cgi->url;

    $url .= '/' . $cgi->param('wdb_action');
}

sub noblanks
{
    my($txt) = @_;

    $txt =~ s/^\s*//;
    $txt =~ s/\s*$//;

    return $txt;
}

__END__
#
#.COPYRIGHT NOTICE
# ==========================================================================
# Copyright Bo Frese Rasmussen 1994 - All Rights Reserved
# Copyright European Southern Observatory(ESO) 1994 - All Rights Reserved
# Copyright Space Telescope - European Coordinating Facility(ST-ECF) 1994 
# - All Rights Reserved
# 
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose is hereby granted without fee, provided
# that the above copyright notice appear in all copies and that both that
# copyright notice, this permission notice, and the following disclaimer
# appear in supporting documentation, and that the names of the copyright
# holders, not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
# 
# BO FRESE RASMUSSEN, EUROPEAN SOUTHERN OBSERVATORY, AND OTHER
# CONTRIBUTORS OF THIS SOFTWARE DISCLAIM ALL WARRANTIES WITH REGARD TO
# THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR ANY PARTICULAR PURPOSE.  IN NO EVENT SHALL BO FRESE
# RASMUSSEN, EUROPEAN SOUTHERN OBSERVATORY OR ANY OTHER CONTRIBUTOR BE
# LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA, OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE, OR OTHER TORTIOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
#------------------------------------------------------------------------------

# ---------------------------------------------------------------------------
# The escape and unescape functions are taken from the wwwurl.pl package
# developed by Roy Fielding <fielding@ics.uci.edu> as part of the Arcadia
# project at the University of California, Irvine. It is distributed
# under the Artistic License (included with your Perl distribution
# files).
# ---------------------------------------------------------------------------
